/* ESP32_ldr_test
 *  
 * Fényérzékeny CdS ellenállás kipróbálása. 
 * A GPIO35 (A7) analóg bemenetre egy fényérzékeny
 * ellenállásosztót kötünk (felső tag egy GL55 fényérzékeny
 * ellenállás, az alsó tag pedig egy 10 kOhmos ellenállás.
 * A leosztott feszültséget a 12 bites felbontásra 
 * és 3,3 V-os méréshatárra konfigurált ADC1-gyel mérjük. 
 * A kapott adatokat 1024 mérésenként átlagoljuk, majd a
 * korábban meghatározott kalibrációs táblázat szerint
 * korrigáljuk. A korrigált adatot feszültségre (V)
 * átszámítjuk, majd a fotoellenállás kOhm-ban mért
 * ellenállását is meghatározzuk és kiíratjuk,
 * A mérések között 2 másodperces szünetet tartunk.
 * 
 * A program Arduino 1.8.15 és ESP32 2.0.0 kiadással tesztelve
 * ADC kalibrálás: https://github.com/e-tinkers/esp32-adc-calibrate
 */
#include <Arduino.h>
#include "LUT.h"                               // Kalibrációs táblázat
#define ADC_PIN 35                             // pin 35 lesz a bemenet
#define VREF   3.30                            // Referencia feszültség
void setup() {
  analogReadResolution(12);                    // 12 bites felbontást
  analogSetAttenuation(ADC_11db);              // Méréshatár 0-3,3 V
  Serial.begin(115200);                        // baudrate = 115200 bit/s
  while (!Serial) {}                           // Várunk a soros portra
}

void loop() {
  uint32_t rawReading = 0 ;
  for (int i = 0; i < 1024; i++) {             // 1024 mérést átlagolunk
    rawReading += analogRead(ADC_PIN);         // az ADC kiolvasása
  }
  rawReading = rawReading >> 10;               // osztás 1024-gyel
  int calibratedReading = ADC_LUT[rawReading]; // a kalibrált érték előkeresése
  float voltage = calibratedReading*VREF/4096; // az osztási pont feszültsége
  Serial.print("Voltage = ");
  Serial.print(voltage,3);
  float Rx = 10000.0;                          // Kiszámítjuk az ellenállást
  if(calibratedReading > 0) {                  // ha nem nulla az ADC eredménye
    Rx = 40960.0f/calibratedReading - 10.0f;   // Rx + 10k = 10k * 4096/NADC
  }    
  Serial.print(" V  Resistance = ");  
  Serial.print(Rx,1);  
  Serial.println(" kOhm"); 
  delay(2000);                                 // Pihenünk egy kicsit...
}
