/* ESP32_TCRT500_test
 *  
 * TCRRT5000 reflexiós fényszenzor kipróbálása. 
 * A GPIO35 (A7) analóg bemenetre közjük a fototoranzisztor
 * kimenő jelét (esetünkben ez az emitter, ami egy 47 kOhmos
 * munkaellenálláson keresztül van GND-re kötve. 
 * ellenállás, az alsó tag pedig egy 10 kOhmos ellenállás.
 * 
 * A GPIO33 digitális kimenet vezérli az IR LED-et.
 * Minen méréskor ki- és bekapcsolt LED-del is végzünk
 * 1024 mérést és az átlagok különbségét tekintjük
 * hasznos jelnek (a többi a szórt háttérfény hatása).
 * 
 * A program Arduino 1.8.15 és ESP32 2.0.0 kiadással tesztelve
 * ADC kalibrálás: https://github.com/e-tinkers/esp32-adc-calibrate
 */
#include <Arduino.h>
#include "LUT.h"                               // Kalibrációs táblázat
#define ADC_PIN    35                          // GPIO35 lesz a bemenet
#define LED_PIN    33                          // GPIO33 digitális kimenet
#define VREF     3.30                          // Referencia feszültség

void setup() {
  analogReadResolution(12);                    // 12 bites felbontást
  analogSetAttenuation(ADC_11db);              // Méréshatár 0-3,3 V
  pinMode(LED_PIN, OUTPUT);                    // legyen digitális kimenet
  Serial.begin(115200);                        // baudrate = 115200 bit/s
  while (!Serial) {}                           // Várunk a soros portra
}

void loop() {
  int sum0 = 0 ;                               // Háttérfény mérés, LED ki
  digitalWrite(LED_PIN,LOW);
  for (int i = 0; i < 1024; i++) {             // 1024 mérést átlagolunk
    sum0 += analogRead(ADC_PIN);               // az ADC kiolvasása
  }
  sum0 = sum0 >> 10;                           // osztás 1024-gyel
  sum0 = ADC_LUT[sum0];                        // a kalibrált érték előkeresése
  
  int sum1 = 0 ;                               // Visszavert fény mérése, LED be
  digitalWrite(LED_PIN,HIGH);
  for (int i = 0; i < 1024; i++) {             // 1024 mérést átlagolunk
    sum1 += analogRead(ADC_PIN);               // az ADC kiolvasása
  }
  digitalWrite(LED_PIN,LOW);
  sum1 = sum1 >> 10;                           // osztás 1024-gyel
  sum1 = ADC_LUT[sum1];                        // a kalibrált érték előkeresése
  int diff = sum1 - sum0;
  if(diff < 0) diff = 0;                       // Negatív érték kiszűrése
  Serial.print(" Összes: ");
  Serial.print(sum1);
  Serial.print(" Háttér: ");
  Serial.print(sum0);
  Serial.print(" Visszavert: ");
  Serial.println(diff);  
  delay(2000);                                  // Pihenünk egy kicsit...
}
