/* ESP32_analog_thermometer
 *  
 * Hőmérés MCP9700 hőmérővel. A GPIO35 analóg bemenetre
 * kötött MCP9700 hőmérő jelét a 12 bites felbontásra 
 * és 3,3 V-os méréshatárra konfigurált ADC1-gyel mérjük. 
 * A kapott adatokat 1024 mérésenként átlagoljuk, majd a
 * korábban meghatározott kalibrációs táblázat szerint
 * korrigáljuk. A korrigált adatot előbb millivoltokra,
 * majd Celsius fokokra átszámítjuk. 
 * A soros porton kiíratjuk az átlagolt nyers adatot, a
 * korrigált adatot, a kiszámolt feszültséget 
 * és az ebből számolt hőmérsékletet is. 
 * A mérések között 2 másodperces szünetet tartunk.
 *  
 * Az MCP9700 és a TMP36 hőmérő nullapontja 500 mV, érzékenysége = 10 mV/C
 * 
 * A program Arduino 1.8.15 és ESP32 2.0.0 kiadással tesztelve
 * ADC kalibrálás: https://github.com/e-tinkers/esp32-adc-calibrate
 */
#include <Arduino.h>
#include "LUT.h"                               // Kalibrációs táblázat
#define ADC_PIN 35                             // pin 35 lesz a bemenet

void setup() {
  analogReadResolution(12);                    // 12 bites felbontást
  analogSetAttenuation(ADC_11db);              // Méréshatár 0-3,3 V
  Serial.begin(115200);                        // baudrate = 115200 bit/s
  while (!Serial) {}                           // Várunk a soros portra
}

void loop() {
  uint32_t rawReading = 0 ;
  for (int i = 0; i < 1024; i++) {             // 1024 mérést átlagolunk
    rawReading += analogRead(ADC_PIN);         // az ADC kiolvasása
  }
  rawReading = rawReading >> 10;               // osztás 1024-gyel
  int calibratedReading = ADC_LUT[rawReading]; // a kalibrált érték előkeresése
  Serial.print(F(" rawReading = "));           // Nyers adat kiírása
  Serial.print(rawReading);
  Serial.print(F(" calibratedReading = "));    // Kalibrált adat kiírása
  Serial.print(calibratedReading);
  int milliVolts=calibratedReading*3300/4096;  // Átszámítás feszültségre (mV)
  float tempC = (milliVolts - 500) / 10.0;     // Átszámítás Celsisus fokokra
  Serial.print(F(" millivolts = "));           // Feszültség kiírása
  Serial.print(milliVolts);
  Serial.print(F(" mV  temperature = "));      // Hőmérséklet kiírása
  Serial.print(tempC, 1);
  Serial.println(F(" °C"));
  delay(2000);                                 // Pihenünk egy kicsit...
}
