/* ESP32_analog_thermometer2
 *  
 * Hőmérés MCP9700 hőmérővel. A GPIO35 analóg bemenetre
 * kötött MCP9700 hőmérő jelét a 12 bites felbontásra 
 * és 3,3 V-os méréshatárra konfigurált ADC1-gyel mérjük. 
 * A kapott adatokat 1024 mérésenként átlagoljuk, majd a
 * korábban meghatározott kalibrációs táblázat szerint
 * korrigáljuk. A korrigált adatot előbb millivoltokra,
 * majd Celsius fokokra átszámítjuk. 
 * A soros porton kiíratjuk a hőmérsékletet, amelyet 
 * a Serial Plotter, vagy az ESP32 LM35 Temperature PC Station
 * Processing alkalmazás segítségével grafikusan is megjeleníthetjük.
 * 
 * Felhasznált források:
 *  - ADC kalibrálás: https://github.com/e-tinkers/esp32-adc-calibrate
 *  - ESP32 LM35 Temperature PC Station (for Processing): 
 *    https://deepbluembedded.com/esp32-temperature-sensor-lm35-interfacing-arduino/
 * 
 * Az MCP9700 és a TMP36 hőmérő nullapontja 500 mV, érzékenysége = 10 mV/C
 * 
 * A program Arduino 1.8.15 és ESP32 2.0.0 kiadással tesztelve
 */

#include <Arduino.h>
#include "LUT.h"
#define ADC_PIN 35

void setup() {
  analogReadResolution(12);
  analogSetAttenuation(ADC_11db); // Méréshatár 0-3,3 V
  Serial.begin(115200);
  while (!Serial) {}
}

void loop() {
  uint32_t rawReading = 0 ;
  for (int i = 0; i < 1024; i++) {
    rawReading += analogRead(ADC_PIN);                // read value from ADC
  }
  rawReading = rawReading>>10;
  int calibratedReading = ADC_LUT[rawReading];        // get the calibrated value from LUT
  int milliVolts = calibratedReading * 3300 / 4096;
  float tempC = (milliVolts - 500) / 10.0;
  Serial.println(tempC,1); 
  delay(1000);
}
