/* ESP32_twighlight_switch
 *  
 * Szürköletkapcsoló CdS ellenállás érzékelővel. 
 * A GPIO35 (A7) analóg bemenetre egy fényérzékeny
 * ellenállásosztót kötünk (felső tag egy GL55 fényérzékeny
 * ellenállás, az alsó tag pedig egy 10 kOhmos ellenállás.
 * A leosztott feszültséget a 12 bites felbontásra 
 * és 3,3 V-os méréshatárra konfigurált ADC1-gyel mérjük. 
 * A kapott adatokat 1024 mérésenként átlagoljuk, majd a
 * korábban meghatározott kalibrációs táblázat szerint
 * korrigáljuk. A korrigált adatot összehasonlítjuk egy
 * előre megadott küszöbértékkel és felkapcsoljuk a 
 * GPIO2 kimenetre kötött LED-et, ha küszöb alatti értéket 
 * mértünk.
 * 
 * A program Arduino 1.8.15 és ESP32 2.0.0 kiadással tesztelve
 * ADC kalibrálás: https://github.com/e-tinkers/esp32-adc-calibrate
 */
#include <Arduino.h>
#include "LUT.h"                               // Kalibrációs táblázat
#define ADC_PIN    35                          // GPIO35 lesz a bemenet
#define LED_PIN     2                          // GPIO2 a digitális kimenet
#define VREF     3.30                          // Referencia feszültség
#define SW_LEVEL 2500                          // Switch level
void setup() {
  analogReadResolution(12);                    // 12 bites felbontást
  analogSetAttenuation(ADC_11db);              // Méréshatár 0-3,3 V
  pinMode(LED_PIN, OUTPUT);                    // legyen digitális kimenet
}

void loop() {
  uint32_t rawReading = 0 ;
  for (int i = 0; i < 1024; i++) {             // 1024 mérést átlagolunk
    rawReading += analogRead(ADC_PIN);         // az ADC kiolvasása
  }
  rawReading = rawReading >> 10;               // osztás 1024-gyel
  int calibratedReading = ADC_LUT[rawReading]; // a kalibrált érték előkeresése
  if (calibratedReading < SW_LEVEL) {
    digitalWrite(LED_PIN,HIGH);                // Lámpa felkapcsolása
  } else {
    digitalWrite(LED_PIN,LOW);                 // Lámpa lekapcsolása
  }
  delay(100);                                  // Pihenünk egy kicsit...
}
