import meter.*;
import processing.serial.*;

Serial port;

Meter M1, M2;

int lf = 10;    // Linefeed in ASCII

void setup() {
  size(850, 350);
  background(0);

  port = new Serial(this, Serial.list()[0], 115200);

  fill(120, 50, 0);
  M1 = new Meter(this, 10, 100);
  // Adjust font color of meter value  +
  M1.setMeterWidth(400);
  M1.setTitleFontSize(20);
  M1.setTitleFontName("Arial bold");
  M1.setTitle("Temperature (°C)");
  M1.setDisplayDigitalMeterValue(true);

  // Meter Scale
  String[] scaleLabelsT = {"0", "10", "20", "30", "40", "50", "60", "70", "80", "90", "100", "110", "120", "130", "140", "150"};
  M1.setScaleLabels(scaleLabelsT);
  M1.setScaleFontSize(18);
  M1.setScaleFontName("Times New Roman bold");
  M1.setScaleFontColor(color(200, 30, 70));

  M1.setArcColor(color(141, 113, 178));
  M1.setArcThickness(10);
  M1.setMaxScaleValue(150);

  M1.setNeedleThickness(3);

  M1.setMinInputSignal(0);
  M1.setMaxInputSignal(150);

  // A second meter for reference
  int mx = M1.getMeterX();
  int my = M1.getMeterY();
  int mw = M1.getMeterWidth();

  M2 = new Meter(this, mx + mw + 20, my);
  M2.setMeterWidth(400);
  M2.setTitleFontSize(20);
  M2.setTitleFontName("Arial bold");
  M2.setTitle("Temperature (°F)");
  M2.setDisplayDigitalMeterValue(true);

  String[] scaleLabelsH = {"20", "40", "60", "80", "100", "120", "140", "160", "180", "200", "220", "240", "260", "280", "300"};
  M2.setScaleLabels(scaleLabelsH);
  M2.setScaleFontSize(18);
  M2.setScaleFontName("Times New Roman bold");
  M2.setScaleFontColor(color(200, 30, 70));

  M2.setArcColor(color(141, 113, 178));
  M2.setArcThickness(10);
  M2.setMaxScaleValue(300);

  M2.setNeedleThickness(3);

  M2.setMinInputSignal(0);
  M2.setMaxInputSignal(300);
}

public void draw() {
  textSize(30);
  fill(255, 255, 255);
  text("ESP32 LM36 Temperature PC Station", 150, 50);

  if (port.available() > 0) {
    String val = port.readStringUntil(lf); 
    if (val != null) {
      float TempC = float(val);
      float TempF = (TempC * 1.8) + 32;
      M1.updateMeter(int(TempC));
      M2.updateMeter(int(TempF));
    }
  }
}
